import org.junit.Test;
import static org.junit.Assert.*;
import java.util.ArrayList;

public class TestWordList2 {
	private void add(WordList list, String[] values) {
		for (String s : values)
			list.insert(s);
	}
	protected WordList createList()	{
		return new TreeWordList();
	}
	protected WordList createList(String values) {
		WordList list = createList();
		if(values.length() > 0)
			add(list, values.split(" "));
		return list;
	}
	@Test
	public void testNode() throws Exception {
		WordNode n = new WordNode("Reinhard");
		assertTrue(n.count == 0 || n.count == 1);
		assertEquals(String.format("%s (%d times)",	n.word, n.count), 
			n.toString());
	}
	@Test
	public void testCount() throws Exception	{
		WordList list = createList(
			"Reinhard Wolfinger Reinhard Bilgeri Reinhard Berger");
		assertEquals(4, list.nOfDifferentWords());
		assertEquals(1, list.getFrequency("Berger"));
		assertEquals(1, list.getFrequency("Bilgeri"));
		assertEquals(3, list.getFrequency("Reinhard"));
		assertEquals(1, list.getFrequency("Wolfinger"));
		assertEquals(1.5, list.meanWordCount());
		assertEquals(3, list.getWords(1).length);
		assertEquals(1, list.getWords(3).length);
		assertEquals(2, list.getWordsStartingWith("b").length);
		list.remove("Reinhard");
		assertEquals(3, list.nOfDifferentWords());
		assertEquals(0, list.getFrequency("Reinhard"));
		assertEquals(0, list.getWords(3).length);
		assertEquals(3.0/3.0, list.meanWordCount(), 0.05);
	}
  @Test
  public void testSort() {
		WordList list = createList();
		String[] names = { 
			"Friedrich", "Berta", "Anton", "Paula", "Caesar", "Zeppelin" };
		String[] sorted = { 
			"Anton", "Berta", "Caesar", "Friedrich", "Paula", "Zeppelin" };
		for(int i=0; i < names.length; i++)
			list.insert(names[i]);
		assertEquals(6, list.nOfDifferentWords());
		for(int i=0; i < names.length; i++)
			assertEquals(1, list.getFrequency(names[i]));
		String[] result = list.getWords(1);
		assertEquals(names.length, result.length);
		for(int i=0; i < names.length; i++)
			assertEquals(0 , result[i].compareToIgnoreCase(sorted[i]));
  }
	@Test
	public void testHamlet() {
		WordList list = createList("To be or not to be");
		assertEquals(2, list.getFrequency("be"));
		assertEquals(4, list.nOfDifferentWords());
		assertEquals(1.5, list.meanWordCount());
		assertEquals(2, list.getWords(2).length);
		assertEquals(0, list.getFrequency("notfound"));
		assertEquals(2, list.getFrequency("BE"));
		assertEquals(0, list.getWords(3).length);
	}
	@Test
	public void testEmpty()	{
		WordList list = createList("");
		assertEquals(0, list.nOfDifferentWords());
		assertEquals(0, list.getWords(1).length);
		assertEquals(0, list.getWordsStartingWith(("")).length);
	}
	protected void readAlice(WordList list)	{
    In.open("alice.txt");
		assertTrue(In.done());
    ArrayList words = new ArrayList();
    char ch = In.peek();
    while (ch != In.eof) {
      if (Character.isLetter(ch)) {
        String ident = In.readIdentifier();
        words.add(ident);
      }
      else {
        In.read();
      }
      ch = In.peek();
    }
		assertEquals(27575, words.size());
    // Test inserting words
    for (int j = 0; j < words.size(); j++)
      list.insert((String)words.get(j));
		In.close();
	}
	@Test	
	public void testAlice()	{
		WordList list = createList();
		readAlice(list);
		assertEquals(2601, list.nOfDifferentWords());
		assertEquals(1648, list.getFrequency("the"));
		assertEquals(417, list.getFrequency("alice"));
		assertEquals(5, list.getFrequency("home"));
		assertEquals(3, list.getFrequency("love"));
		assertEquals(0, list.getFrequency("humpty"));
		assertEquals(10.6, list.meanWordCount(), 0.05);
		assertEquals(179, list.getWordsStartingWith("t").length);
		assertEquals(99, list.getWordsStartingWith("e").length);
		assertEquals(142, list.getWordsStartingWith("a").length);
	}
}
